#ifndef stracciatella_h
#define stracciatella_h

#include <cstdarg>
#include <cstdint>
#include <cstdlib>
#include <new>

static const uint32_t ENTRY_BYTES = 280;

static const uint32_t HEADER_BYTES = 532;

static const uint64_t UNIX_EPOCH_AS_FILETIME = 116444736000000000;

/// Enum used to specify scaling quality for scaling up graphics
enum class ScalingQuality {
  /// Use linear interpolation
  LINEAR,
  /// Use nearest neighbor interpolation
  NEAR_PERFECT,
  /// Scale up to the nearest multiple of 640x480 and use nearest neighbor interpolation
  PERFECT,
};

/// Enum for the vanilla game version that is used to run the game
enum class VanillaVersion {
  DUTCH,
  ENGLISH,
  FRENCH,
  GERMAN,
  ITALIAN,
  POLISH,
  RUSSIAN,
  RUSSIAN_GOLD,
};

/// Struct that is used to store the engines configuration parameters
struct EngineOptions;

/// Thread safe library database.
struct LibraryDB;

/// Library file.
/// Provides access to the data of a library entry through Read and Seek.
struct LibraryFile;

struct VecCString;

extern "C" {

/// Adds a library to the end of the library database.
/// Returns true is successful, false otherwise.
/// Sets the rust error.
bool LibraryDB_AddLibrary(LibraryDB *ldb, const char *data_dir, const char *library);

/// Destructor.
/// The caller is no longer responsible for the memory of the library database.
void LibraryDB_Delete(LibraryDB *ldb);

/// Constructor.
/// The caller is responsible for the memory of the library database.
LibraryDB *LibraryDB_New();

/// Closes a library database file.
/// The caller is no longer responsible for the library file memory.
void LibraryFile_Close(LibraryFile *file);

/// Gets the current position in a library database file.
uint64_t LibraryFile_GetPos(LibraryFile *file);

/// Gets the size of a library database file.
uint64_t LibraryFile_GetSize(LibraryFile *file);

/// Opens a library database file for reading.
/// Returns the file on success and null on error.
/// The caller is responsible for the library file memory.
/// Sets the rust error.
LibraryFile *LibraryFile_Open(LibraryDB *ldb, const char *path);

/// Reads from a library database file.
/// Sets the rust error.
bool LibraryFile_Read(LibraryFile *file, uint8_t *buffer, size_t buffer_length);

/// Seeks a library database file.
/// `from` expects FileSeekMode values (see src/sgp/SGPFile.h)
/// Sets the rust error.
bool LibraryFile_Seek(LibraryFile *file, int64_t distance, int from);

/// Returns true if it was able to find path relative to base.
/// Makes caseless searches one component at a time.
bool check_if_relative_path_exists(const char *base, const char *path, bool caseless);

void clear_mods(EngineOptions *ptr);

EngineOptions *create_engine_options(const char *const *array, size_t length);

char *find_ja2_executable(const char *launcher_path_ptr);

/// Finds a path relative to the stracciatella home directory.
/// If path is null, it finds the stracciatella home directory.
/// If test_exists is true, it makes sure the path exists.
char *find_path_from_stracciatella_home(const char *path, bool test_exists);

void free_engine_options(EngineOptions *ptr);

void free_rust_string(char *s);

/// Returns a list of available mods.
VecCString *get_available_mods();

float get_brightness(const EngineOptions *ptr);

/// Returns the path to game.json.
char *get_game_json_path();

char *get_mod(const EngineOptions *ptr, uint32_t index);

uint32_t get_number_of_mods(const EngineOptions *ptr);

uint16_t get_resolution_x(const EngineOptions *ptr);

uint16_t get_resolution_y(const EngineOptions *ptr);

VanillaVersion get_resource_version(const EngineOptions *ptr);

char *get_resource_version_string(VanillaVersion version);

/// Returns a copy of the thread local rust error or null.
/// The caller is responsible for the memory.
char *get_rust_error();

ScalingQuality get_scaling_quality(const EngineOptions *ptr);

char *get_scaling_quality_string(ScalingQuality quality);

char *get_stracciatella_home(const EngineOptions *ptr);

char *get_vanilla_game_dir(const EngineOptions *ptr);

/// Guesses the resource version from the contents of the game directory.
/// Returns a VanillaVersion value if it was sucessful, -1 otherwise.
/// If log_ptr is not null, it will receive a rust string with the log of the guess attempt.
int guess_resource_version(const char *gamedir, char **log_ptr);

void push_mod(EngineOptions *ptr, const char *name);

void set_brightness(EngineOptions *ptr, float brightness);

void set_resolution(EngineOptions *ptr, uint16_t x, uint16_t y);

void set_resource_version(EngineOptions *ptr, VanillaVersion res);

void set_scaling_quality(EngineOptions *ptr, ScalingQuality scaling_quality);

void set_start_in_fullscreen(EngineOptions *ptr, bool val);

void set_start_without_sound(EngineOptions *ptr, bool val);

void set_vanilla_game_dir(EngineOptions *ptr, const char *game_dir_ptr);

bool should_run_editor(const EngineOptions *ptr);

bool should_run_unittests(const EngineOptions *ptr);

bool should_show_help(const EngineOptions *ptr);

bool should_start_in_debug_mode(const EngineOptions *ptr);

bool should_start_in_fullscreen(const EngineOptions *ptr);

bool should_start_in_window(const EngineOptions *ptr);

bool should_start_without_sound(const EngineOptions *ptr);

void vec_c_string_delete(VecCString *vec);

char *vec_c_string_get(VecCString *vec, size_t index);

size_t vec_c_string_len(VecCString *vec);

bool write_engine_options(EngineOptions *ptr);

} // extern "C"

#endif // stracciatella_h
